@@ -39,9 +39,12 @@ import re
 import sys
 
 import logzero
+import uproot
 from logzero import logger as log
 
-from . import pid_data, utils
+from . import binning, markdown_table, pid_data, utils
+
+uproot.open.defaults["xrootd_handler"] = uproot.source.xrootd.XRootDSource
 
 try:
     from .version import version  # type: ignore
@@ -49,6 +52,50 @@ except ImportError:
     version = "N/A"
 
 
+class ListValidAction(argparse.Action):
+    """Class that overrides required parameters and prints valid configs."""
+
+    def __call__(self, parser, namespace, values, option_string=None):
+
+        if values == "configs" or values.endswith(".json"):
+            header = ["Sample", "Magnet", "Particle"]
+            table = markdown_table.MarkdownTable(header)
+
+            # Print configs from the default samples.json if no file specified
+            if values == "configs":
+                values = None
+
+            for entry in pid_data.get_calibration_samples(values).keys():
+                try:
+                    sample, magnet, particle = entry.split("-")
+                    magnet = magnet[3:].lower()
+                    table.add_row([sample, magnet, particle])
+                except ValueError:
+                    # Skip group entries like "Turbo18-MagUp"
+                    pass
+
+            table.print()
+
+        elif values == "aliases":
+            table_pid = markdown_table.MarkdownTable(["Alias", "PID Variable"])
+            for alias, var in pid_data.pid_aliases.items():
+                table_pid.add_row([alias, var])
+            table_pid.print()
+
+            print()
+
+            table_bin = markdown_table.MarkdownTable(["Alias", "Binning Variable"])
+            for alias, var in pid_data.bin_aliases.items():
+                table_bin.add_row([alias, var])
+            table_bin.print()
+
+        else:
+            log.error(f"'{values}' is not a known keyword for list-valid")
+            raise KeyError
+
+        parser.exit()
+
+
 def decode_arguments(args):
     """Decode CLI arguments."""
     parser = argparse.ArgumentParser(
@@ -103,6 +150,11 @@ def decode_arguments(args):
         dest="bin_vars",
         required=True,
     )
+    parser.add_argument(
+        "-g",
+        "--binning-file",
+        help="file where new/alternative binnings are defines",
+    )
     parser.add_argument(
         "-o",
         "--output-dir",
@@ -111,6 +163,12 @@ def decode_arguments(args):
     )
     parser.add_argument(
         "-l",
+        "--list-valid",
+        action=ListValidAction,
+        help="list all valid [configs, aliases]",
+    )
+    parser.add_argument(
+        "-d",
         "--local-dataframe",
         help="(debug) read a calibration DataFrame from file",
     )
@@ -165,29 +223,36 @@ def make_eff_hists(config: dict) -> None:
         re.sub(pattern, "", pid_cut) for pid_cut in config["pid_cuts"]
     ]
 
+    config["version"] = version
     log.info("Running PIDCalib2 make_eff_hists with the following config:")
     utils.log_config(config)
 
     output_dir = pathlib.Path(config["output_dir"])
     output_dir.mkdir(parents=True, exist_ok=True)
 
-    calibration_prefix = "probe"
-    branch_names = pid_data.get_relevant_branch_names(
-        calibration_prefix, config["pid_cuts"], config["bin_vars"], config["cuts"]
-    )
-    log.info(f"Branches to be read: {branch_names}")
+    if config["binning_file"] is not None:
+        binning.load_binnings(config["binning_file"])
+
+    # Check that all binnings exist before reading files
+    for bin_var in config["bin_vars"]:
+        bin_edges = binning.get_binning(config["particle"], bin_var, verbose=True)
+        log.debug(f"{bin_var} binning: {bin_edges}")
 
     df_total = None
     if config["local_dataframe"]:
+        branch_names = pid_data.get_relevant_branch_names(
+            config["pid_cuts"], config["bin_vars"], config["cuts"]
+        )
         df_total = pid_data.dataframe_from_local_file(
             config["local_dataframe"], list(branch_names)
         )
     else:
+        calib_sample = {}
         if config["file_list"]:
             with open(config["file_list"]) as f_list:
-                eos_paths = f_list.read().splitlines()
+                calib_sample["files"] = f_list.read().splitlines()
         else:
-            eos_paths = pid_data.get_file_list(
+            calib_sample = pid_data.get_calibration_sample(
                 config["sample"],
                 config["magnet"],
                 config["particle"],
@@ -195,24 +260,49 @@ def make_eff_hists(config: dict) -> None:
                 config["max_files"],
             )
         tree_paths = pid_data.get_tree_paths(config["particle"], config["sample"])
+        log.debug(f"Trees to be read: {tree_paths}")
 
-        log.info(f"{len(eos_paths)} calibration files from EOS will be processed")
-        for path in eos_paths:
+        # If there are hard-coded cuts, the variables must be included in the
+        # branches to read.
+        cuts = config["cuts"]
+        if "cuts" in calib_sample:
+            if cuts is None:
+                cuts = []
+            cuts += calib_sample["cuts"]
+
+        branch_names = pid_data.get_relevant_branch_names(
+            config["pid_cuts"], config["bin_vars"], cuts
+        )
+        log.info(f"Branches to be read: {branch_names}")
+        log.info(
+            f"{len(calib_sample['files'])} calibration files from EOS will be processed"
+        )
+        for path in calib_sample["files"]:
             log.debug(f"  {path}")
-        df_total = pid_data.calib_root_to_dataframe(eos_paths, tree_paths, branch_names)
+
+        df_total = pid_data.calib_root_to_dataframe(
+            calib_sample["files"], tree_paths, branch_names
+        )
+
+        binning_range_cuts = []
+        for bin_var in config["bin_vars"]:
+            bin_edges = binning.get_binning(config["particle"], bin_var, verbose=True)
+            binning_range_cuts.append(
+                f"{bin_var} > {bin_edges[0]} and {bin_var} < {bin_edges[-1]}"
+            )
+        log.debug(f"Applying binning range cuts: {binning_range_cuts}'")
+        utils.apply_cuts(df_total, binning_range_cuts)
+
+        if "cuts" in calib_sample:
+            log.debug(f"Applying hard-coded cuts: {calib_sample['cuts']}'")
+            utils.apply_cuts(df_total, calib_sample["cuts"])
+
         # df_total.to_pickle("local_dataframe.pkl")
         # df_total.to_csv("local_dataframe.csv")
 
     if config["cuts"]:
-        cut_string = " & ".join(config["cuts"])
-        log.debug(f"Applying generic cuts: '{cut_string}'")
-        num_before = df_total.shape[0]
-        df_total.query(cut_string, inplace=True)
-        num_after = df_total.shape[0]
-        log.debug(
-            f"{num_after}/{num_before} ({num_after/num_before:.1%}) events "
-            "passed generic cuts"
-        )
+        log.debug(f"Applying user cuts: '{config['cuts']}'")
+        utils.apply_cuts(df_total, config["cuts"])
 
     eff_hists = utils.create_eff_histograms(
         df_total, config["particle"], config["pid_cuts"], config["bin_vars"]
