@@ -10,7 +10,7 @@
 ###############################################################################
 
 import re
-from typing import Dict, List, Union
+from typing import Dict, List, Tuple, Union
 
 import boost_histogram as bh
 import numpy as np
@@ -36,17 +36,8 @@ def make_hist(
 
     # Loop over bin dimensions and define the axes
     for bin_var in bin_vars:
-        if (
-            particle not in binning.binnings
-            or bin_var not in binning.binnings[particle]
-        ):
-            log.error(f"No binning defined for particle {particle} variable {bin_var}")
-
-        axis_list.append(
-            bh.axis.Variable(
-                binning.binnings[particle][bin_var], metadata={"name": bin_var}
-            )
-        )
+        bin_edges = binning.get_binning(particle, bin_var)
+        axis_list.append(bh.axis.Variable(bin_edges, metadata={"name": bin_var}))
         vals = df[bin_var].values
         vals_list.append(vals)
 
@@ -83,7 +74,7 @@ def create_eff_histograms(
     zero_bins = np.count_nonzero(hists["total"].view(flow=False) == 0)
     if zero_bins:
         log.warning(f"There are {zero_bins} empty bins in the total histogram!")
-        log.warning(hists["total"].view(flow=False))
+        log.debug(hists["total"].view(flow=False))
 
         # Replace zeros with NaNs which suppresses duplicate Numpy warnings
         hist_total_nan = hists["total"].view()
@@ -207,14 +198,19 @@ def add_efficiencies(
     df_new.dropna(inplace=True)
 
     df_new["PIDCalibEff"] = 1
-    df_new["PIDCalibErr2"] = 0
+    df_new["PIDCalibRelErr2"] = 0
 
     for prefix in prefixes:
         efficiency_table = eff_hists[prefix]["eff"].view().flatten()  # type: ignore
         error_table = (
             create_error_histogram(eff_hists[prefix]).view().flatten()  # type: ignore
         )
-        np.nan_to_num(efficiency_table, False)  # Replicate old PIDCalib's behavior
+        # The original PIDCalib assigned bins with no events in the total
+        # histogram an efficiency of zero. This should not come up often and the
+        # user should be warned about it. In any case it does not seem right -
+        # we assign the bin a NaN. This might cause slightly different results
+        # when using a sample and binning that lead to such empty bins.
+        # np.nan_to_num(efficiency_table, False)  # Replicate old PIDCalib's behavior
 
         # Assign efficiencies by taking the efficiency value from the relevant bin
         df_new[f"{prefix}_PIDCalibEff"] = np.take(
@@ -225,12 +221,17 @@ def add_efficiencies(
             error_table, df_new[f"{prefix}_PIDCalibBin"]
         )
         df_new["PIDCalibEff"] = df_new["PIDCalibEff"] * df_new[f"{prefix}_PIDCalibEff"]
-        df_new["PIDCalibErr2"] += df_new[f"{prefix}_PIDCalibErr"] ** 2
+        df_new["PIDCalibRelErr2"] += (
+            df_new[f"{prefix}_PIDCalibErr"] / df_new[f"{prefix}_PIDCalibEff"]
+        ) ** 2
 
-    df_new["PIDCalibErr"] = np.sqrt(df_new["PIDCalibErr2"])  # type: ignore
-    df_new.drop(columns=["PIDCalibErr2"], inplace=True)
+    df_new["PIDCalibErr"] = np.sqrt(df_new["PIDCalibRelErr2"])  # type: ignore
+    for prefix in prefixes:
+        df_new["PIDCalibErr"] *= df_new[f"{prefix}_PIDCalibEff"]
 
-    df_new = pd.concat([df_new, df_nan]).sort_index()
+    df_new.drop(columns=["PIDCalibRelErr2"], inplace=True)
+
+    df_new = pd.concat([df_new, df_nan]).sort_index()  # type: ignore
     log.debug("Particle efficiencies assigned")
 
     num_outside_range = len(df_nan.index)
@@ -256,7 +257,7 @@ def create_hist_filename(
     whitespace = re.compile(r"\s+")
     cut = re.sub(whitespace, "", pid_cut)
 
-    return f"effhists_{sample}_{magnet}_{particle}_{cut}_{'-'.join(bin_vars)}.pkl"
+    return f"effhists-{sample}-{magnet}-{particle}-{cut}-{'.'.join(bin_vars)}.pkl"
 
 
 def binomial_uncertainty(
@@ -302,3 +303,14 @@ def create_error_histogram(eff_hists: Dict[str, bh.Histogram]) -> bh.Histogram:
     err_histo = eff_hists["passing"].copy()
     err_histo[...] = uncertainty
     return err_histo
+
+
+def apply_cuts(df: pd.DataFrame, cuts: List[str]) -> Tuple[int, int]:
+    cut_string = " and ".join(cuts)
+    num_before = df.shape[0]
+    df.query(cut_string, inplace=True)
+    num_after = df.shape[0]
+    log.debug(
+        f"{num_after}/{num_before} ({num_after/num_before:.1%}) events " "passed cuts"
+    )
+    return num_before, num_after
